from os import path
from pathlib import Path

from bs4 import BeautifulSoup
from requests import get


class GoogleSheetsDownloader:
    """
    The class is used for downloading the Google Sheet spreadsheets by its link.
    When creating an object, it is necessary to pass the Google Sheet URL to the class.
    Also, as the second optional parameter,
    you can pass the format in which you want to download the spreadsheet.
    Supported formats: 'xlsx', 'ods', 'pdf', 'html', 'csv', 'tsv'.
    -------------------------------------------------------------------------------------
    It's 'download_file' method to download the spreadsheet.
    It's 'get_downloaded_file_name' method to get the name of the downloaded spreadsheet.
    """
    __last_downloaded_file = ''
    __extensions = (
        'xlsx',
        'ods',
        'pdf',
        'html',
        'csv',
        'tsv',
    )

    def __init__(self, sheet_url: str, extension='csv'):
        self.__url = self.__get_valid_url(url=sheet_url)
        self.__sheet_id = self.__get_id(url=self.__url)
        self.__extension = self.__get_extension(extension)
        self.__export_format = f'/export?format={self.__extension}&id='
        self.__download_url = self.__url + self.__export_format + self.__sheet_id

    def __get_extension(self, extension: str) -> str:
        if extension in self.__extensions:
            return 'zip' if extension == 'html' else extension
        else:
            message = f'It\'s impossible to convert this sheet into \'{extension}\' file\n' \
                      f'Use some of the next extensions {self.__extensions}'
            raise TypeError(message)

    def get_extension(self):
        return self.__extension

    @staticmethod
    def __get_name(url: str) -> str:
        try:
            response = get(url)
            soup = BeautifulSoup(response.text, 'html.parser')
            return soup.find(id='docs-title-input-label-inner').text
        except AttributeError:
            return 'Sheet'

    @staticmethod
    def __get_valid_url(url: str) -> str:
        if not isinstance(url, str):
            raise TypeError('sheet_url must be str type')
        edit_ending = '/edit#gid=0'
        if url.endswith(edit_ending):
            url = url.replace(edit_ending, '')
        return url

    @staticmethod
    def __get_id(url: str) -> str:
        sheet_id = url[url.rfind('/') + 1:]
        if sheet_id == -1:
            raise ValueError(f'{url}\nIt isn\'t a Google Sheets URL')
        return sheet_id

    def get_downloaded_file_name(self) -> str:
        return self.__last_downloaded_file

    def download_file(self, directory='') -> None:
        print('Downloading the spreadsheet...')

        filename = self.__get_name(url=self.__url)
        number = 1
        full_filename = Path(directory, f'{filename}.{self.__extension}')

        while path.isfile(full_filename):
            number += 1
            full_filename = Path(directory, f'{filename + str(number)}.{self.__extension}')

        self.__last_downloaded_file = full_filename.name

        with open(full_filename, 'wb') as file:
            file.write(get(url=self.__download_url).content)
            print(f'The download of the {self.__last_downloaded_file} file is complete')
